<?php

declare(strict_types=1);

namespace Drupal\menu_tree\Hook;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\menu_tree\MenuTreeItems;
use Drupal\menu_tree\NodeFormSubmitHandler;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class Hooks.
 */
class Hooks implements ContainerInjectionInterface {

  use StringTranslationTrait;

  protected ConfigFactoryInterface $configFactory;
  protected NodeFormSubmitHandler $nodeFormSubmitHandler;
  protected MenuTreeItems $menuTreeItems;

  /**
   * Hooks constructor.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    NodeFormSubmitHandler $node_form_submit_handler,
    MenuTreeItems $menu_tree_items
  ) {
    $this->configFactory = $config_factory;
    $this->nodeFormSubmitHandler = $node_form_submit_handler;
    $this->menuTreeItems = $menu_tree_items;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) : static {
    return new static(
      $container->get('config.factory'),
      $container->get('menu_tree.node_form_submit_handler'),
      $container->get('menu_tree.items')
    );
  }

  /**
   * Implements hook_help().
   */
  #[Hook('help')]
  function help($route_name, RouteMatchInterface $route_match) : string|\Stringable|array|null {
    if ($route_name == 'help.page.menu_tree') {
      $output = '';
      $output .= '<h3>' . t('About') . '</h3>';
      $output .= '<p>' . t('This modules replaces the standard widget for selecting Parent link on node add and edit forms with a tree widget.') . '</p>';
      $output .= '<h3>' . t('Supporting organizations:') . '</h3>';
      $output .= '<p>' . t('<a href="https://www.drupal.org//happiness">Happiness</a>') . '</p>';
      return $output;
    }
    return NULL;
  }

  /**
   * Implements hook_form_BASE_FORM_ID_alter().
   *
   * This function modifies the node form elements based on the configuration
   * of the 'menu_tree' module. It uses the settings for bundles and available
   * menus to determine the alterations needed for the menu-related form
   * elements. When applicable, it hides the default menu parent and weight
   * elements and adds a tree selector for menu placement.
   */
  #[Hook('form_node_form_alter')]
  function nodeFormAlter(&$form, FormStateInterface $form_state, $form_id) : void {
    /** @var \Drupal\node\NodeInterface $node */
    $node = $form_state->getFormObject()->getEntity();
    $bundles = $this->configFactory->get('menu_tree.settings')->get('bundles');

    // No bundles configured, do nothing.
    if (empty($bundles)) {
      return;
    }

    // Should we alter this form?
    if (in_array($node->bundle(), $bundles)) {
      // Get available menus from node.type.<bundle>.yml.
      $available_menus = $this->configFactory->get('node.type.' . $node->bundle())->get('third_party_settings.menu_ui.available_menus');

      // No menus configured for the bundle, do nothing.
      if (empty($available_menus)) {
        return;
      }

      // Hide menu parent selector and weight elements.
      $form['menu']['link']['menu_parent']['#type'] = 'hidden';
      $form['menu']['link']['weight']['#type'] = 'hidden';

      $defaults = menu_ui_get_menu_link_defaults($node);
      $exclude = $defaults['menu_name'] . ':' . $defaults['id'];

      // Get menus.
      $menus = [];
      foreach ($available_menus as $id) {
        $menus[] = $this->menuTreeItems->getLinks($id);
      }

      // Add the menu tree component.
      $form['menu']['link']['menu_tree'] = [
        '#type' => 'component',
        '#component' => 'menu_tree:menu-tree',
        '#props' => [
          'menus' => $menus,
          'exclude' => $exclude,
        ],
        '#title' => $this->t('Parent link'),
        '#theme_wrappers' => ['form_element'],
        '#cache' => [
          'max-age' => 0,
        ],
      ];

      $form['menu']['link']['prev_sibling'] = [
        '#type' => 'hidden',
        '#value' => '',
      ];

      $form['menu']['link']['next_sibling'] = [
        '#type' => 'hidden',
        '#value' => '',
      ];

      // Add submit handler.
      foreach (array_keys($form['actions']) as $action) {
        if ($action != 'preview' && isset($form['actions'][$action]['#type']) && $form['actions'][$action]['#type'] === 'submit') {
          $form['actions'][$action]['#submit'][] = [$this->nodeFormSubmitHandler, 'handleFormSubmit'];
        }
      }
    }
  }

}
